"""
Generates graphs

Copyright 2020, University of Freiburg

Philipp Schneider <philipp.schneider@cs.uni-freiburg.de>
"""

import math
import random

import networkx as nx


def random_adj_matrix(n):
    """
        Generate a random adjacency matrix with edge weights
        from euklidian distances of n random 2d-points
    """

    points = [(n * random.uniform(0, 1),
              n * random.uniform(0, 1)) for i in range(n)]

    matrix = [[round(dist(points[i], points[j]), 2)
              for j in range(n)] for i in range(n)]

    return matrix


def dist(a, b):
    """
        Returns the euklidian distance between two 2d-points

        Unit-tests:
            >>> dist((1,2),(2,2))
            1.0
            >>> dist((1,1),(1,3))
            2.0
            >>> round(dist((0,0),(1,1)),2)==round(math.sqrt(2),2)
            True
            >>> round(dist((0,0),(50,50)),2)==round(50*math.sqrt(2),2)
            True
    """

    square_sum = (a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2

    return math.sqrt(square_sum)


def adj_matrix_to_file(n, filename):
    """
        generate and write a rnadom adj_matrix as text file
    """

    adj_matrix = random_adj_matrix(n)

    with open(filename, 'w') as input:
        input.write(str(n) + '\n')
        for i in range(n):
            for j in range(n):
                input.write(str(adj_matrix[i][j]) + ' ')
            input.write('\n')


def random_dag(n, m, indeg):
    """
        Generate a random Directed Acyclic Graph (DAG)
        with a given number of nodes and edges.
    """
    g = nx.DiGraph()
    for i in range(n):
        g.add_node(i)
    while m > 0:
        a = 0
        b = 0
        while b == a or g.has_edge(a, b) or g.in_degree(b) >= indeg:
            a = random.randint(0, n-1)
            b = random.randint(0, n-1)
        g.add_edge(a, b)
        if nx.is_directed_acyclic_graph(g):
            m -= 1
        else:
            # we closed a loop!
            g.remove_edge(a, b)
    return g


def graph_to_file(n, m, indeg, filename):

    g = random_dag(n, m, indeg)

    with open(filename, 'w') as input:
        input.write(str(n) + '\n')
        for e in g.edges:
            input.write(str(e[0]) + ' ' + str(e[1]) + '\n')
